library text_field_options_values;

import 'package:flutter/material.dart';

///TexFieldOptions
///Utiliza opciones de unidades  sufijos, dentro del mismo textfield
/// * Con la variable [options] envia una lista de [String] la cual colocará al final del input
///   Cada [String] Sera seleccionable, el valor inicial sera el primer valor de la lista.
///   En caso de que no envies una lista, el valor será [null]
/// * Con [validator] podrémos validar el texto e incluso la opcion seleciconada
///
/// * Con [onFieldSubmitted] Prdrémos almacenar las variable utilizadas
class TextFieldOptions extends StatefulWidget {
  final List<String> options;
  final Function? validator;
  final String textInitial;
  final String optionInitial;
  final BoxDecoration? decorationSelectedOption;
  final BoxDecoration? decorationUnselectedOption;
  final BoxDecoration? decoration;
  final TextStyle? textStyleSelectedOption;
  final TextStyle? textStyleUnselectedOption;
  final EdgeInsetsGeometry? margin;
  final TextStyle inputStyle = const TextStyle(fontSize: 15);
  
  const TextFieldOptions(
      {this.textInitial = "",
      this.optionInitial = "",
      this.decorationSelectedOption,
      this.decorationUnselectedOption,
      this.decoration,
      this.textStyleSelectedOption,
      this.textStyleUnselectedOption,
      this.margin,
      this.options = const [],
      this.validator,
      Key? key})
      : super(key: key);

  @override
  _TextFieldOptionsState createState() => _TextFieldOptionsState();
}

class _TextFieldOptionsState extends State<TextFieldOptions> {
  FocusNode focusNode = FocusNode();
  var isFocused = false;

  var optionSelcted = "";
  TextEditingController controllerField = InputSufixController();
  @override
  initState() {
    super.initState();
    focusNode.addListener(() {
      setState(() {
        isFocused = focusNode.hasFocus;
      });
    });
    controllerField.text = widget.textInitial;

    if (widget.optionInitial.isNotEmpty) {
      var optionExist = widget.options
          .firstWhere((element) => element == widget.optionInitial);
      if (optionExist.isNotEmpty) {
        optionSelcted = optionExist;
        return;
      }
    }
    optionSelcted = widget.options.isNotEmpty ? widget.options[0] : "";
  }

  //Obtenemos el item opcion
  Widget getNewOption(String textOption) {
    //Asignamos la varible style Unselected en caso de que sea null
    BoxDecoration styleUnselectedBox = widget.decorationUnselectedOption != null
        ? widget.decorationUnselectedOption!
        : const BoxDecoration(
            color: Colors.white,
            borderRadius: BorderRadius.all(Radius.circular(5)));

    //Asignamos la varible style Unselected en caso de que sea null
    BoxDecoration styleSelectedBox = widget.decorationSelectedOption != null
        ? widget.decorationSelectedOption!
        : const BoxDecoration(
            color: Colors.blue,
            borderRadius: BorderRadius.all(Radius.circular(5)));
    //Estilo para el texto de la opcion seleccionada
    TextStyle textStyleSelected = widget.textStyleSelectedOption != null
        ? widget.textStyleSelectedOption!
        : const TextStyle(
            color: Colors.black, fontSize: 15, fontWeight: FontWeight.w500);
    //Estilo para el texto de la opcion sin seleccionar
    TextStyle textStyleUnselected = widget.textStyleUnselectedOption != null
        ? widget.textStyleUnselectedOption!
        : const TextStyle(
            color: Colors.black, fontSize: 15, fontWeight: FontWeight.w500);

    ///Se utiliza una columna para que se adapte el container al padre
    ///Utilizando el expanded, de tal manera adaptamos el padre al tamaño.
    return Column(children: [
      Expanded(
        ///Agregamos un gesto de click al contenedor par  detectar cuando cambio la opcion
        child: GestureDetector(
          onTap: () {
            debugPrint(textOption);

            ///Actualizamos la opcion  seleccionada
            setState(() {
              optionSelcted = textOption;
              //Seteamos el focus en el input
              focusNode.requestFocus();
            });
          },
          child: Container(
              //Asignamos la decoracion segun si esta seleccinada o no
              decoration: textOption == optionSelcted
                  ? styleSelectedBox
                  : styleUnselectedBox,
              margin: const EdgeInsets.all(3),
              padding: const EdgeInsets.all(5),
              child: Center(
                child: Text(textOption,
                    //Asignamos el estilo del texto segun si esta seleccinada la opcion o no
                    style: textOption == optionSelcted
                        ? textStyleSelected
                        : textStyleUnselected),
              )),
        ),
      ),
    ]);
  }

  @override
  Widget build(BuildContext context) {
    //Estilo del contenedor principal
    BoxDecoration boxDecor = widget.decoration != null
        ? widget.decoration!
        : BoxDecoration(            
            color: Colors.white,
            border: Border.all(
                color: isFocused? Colors.blue : Colors.grey, width: 1.5, style: BorderStyle.solid),
            borderRadius: const BorderRadius.all(Radius.circular(10)),
          );

    ///Inicializamos el contenedor principal
    return Container(
      margin: widget.margin ?? const EdgeInsets.all(10),
      padding: const EdgeInsets.all(1),
      height: widget.inputStyle.fontSize! * 2.5,
      decoration: boxDecor,
      child: Row(  
        crossAxisAlignment: CrossAxisAlignment.center,      
        children: [
          //input
          Expanded(
            child: TextFormField(  
              focusNode: focusNode,
              decoration: const InputDecoration(
                border: InputBorder.none,
                contentPadding: EdgeInsets.symmetric(vertical: 0, horizontal: 10), //Change this value to custom as you like
                isDense: true, // and add this line
                ),                        
              controller: controllerField,
              onSaved: (String? value) {
                // This optional block of code can be used to run
                // code when the user saves the form.
              },
              validator: (String? value) {
                if (widget.validator != null) {
                  return widget.validator!(value, optionSelcted);
                }
                return null;
              },
              style: widget.inputStyle,
              textAlign: TextAlign.start,
              textAlignVertical: TextAlignVertical.center,
            ),
          ),
          Visibility(
            visible: widget.options.length > 0,
            child: const VerticalDivider(
              width: 10,
              thickness: 1.5,
              color: Colors.grey,
            ),
          ),
          Row(
            children: widget.options.map((item) => getNewOption(item)).toList(),
          )
          //opciones
        ],
      ),
    );
  }
}

class InputSufixController extends TextEditingController {
  InputSufixController();
  TextEditingController cont = TextEditingController();
  String optionSelected = "";
  set setOption(String op) {
    optionSelected = op;
  }

  get option => optionSelected;

  get controller => cont;
}
